/*$Id: l_dispatcher.h,v 26.7 2006/11/22 07:17:06 al Exp $ -*- C++ -*-
 * Copyright (C) 2006 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *------------------------------------------------------------------
 * dispatcher -- for dynamically loaded modules
 */
#ifndef L_DISPATCHER_H
#define L_DISPATCHER_H
#include "l_stlextra.h"
#include "ap.h"
/*--------------------------------------------------------------------------*/
template <class T>
class DISPATCHER {
private:
  std::map<std::string, const T*> _map;
  bool _initialized;
public:
  explicit DISPATCHER() 
    :_initialized(false)
  {
    _initialized = true;
  }

  ~DISPATCHER() {_initialized = false;}

  void install(const std::string& s, const T* p) {
    if (!_initialized) {
      std::cerr << "incorrect link order\n";
      abort();
    }
    if (_map[s]) {
      error(bWARNING, s + ": already installed, replacing\n");
    }else{
    }
    _map[s] = p;
  }

  void uninstall(const std::string& s) {
    assert(_initialized);
    if (_map[s]) {
      _map[s] = NULL;
    }else{
      error(bWARNING, s + ": not installed, doing nothing\n");
    }
  }

  const T* operator[](const std::string& s) {
    return _map[s];
  }

  const T* operator[](CS& cmd) {
    int here = cmd.cursor();
    std::string s;
    cmd >> s;
    notstd::to_lower(&s);
    //------------------------
    const T* p = (*this)[s];
    //------------------------
    if (!p) {
      cmd.reset(here);
    }else{
    }
    return p;
  }

  class INSTALL {
  private:
    const std::string _name;
    DISPATCHER<T>* _d;
  public:
    INSTALL(DISPATCHER<T>* d, const std::string& name, const T* p) :
      _name(name),
      _d(d) 
    {
      assert(_d);
      assert(p);
      _d->install(_name, p);
    }
    
    ~INSTALL() {
      _d->uninstall(_name);
    }
  };
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
